import re
from typing import Optional, Dict, Tuple
from palm.plugins.dbt.local_user_lookup import local_user_lookup

""" Shared DBT utilities to build out common CLI options """


def shell_options(command_name: str, **kwargs):
    # pop off unused kwargs caused by use of importlib in palm
    kwargs.pop("ctx")

    if kwargs["fast"]:
        for k in ("fast", "select"):
            kwargs.pop(k, None)
        return _short_cycle(command_name, **kwargs)
        return
    kwargs.pop("fast")
    return _long_cycle(command_name, **kwargs)


def _short_cycle(
    cmd: str,
    persist: Optional[bool] = False,
    no_fail_fast: Optional[bool] = False,
    full_refresh: Optional[bool] = False,
    no_seed: Optional[bool] = False,
    models: Optional[Tuple] = (),
    macros: Optional[Tuple] = (),
) -> str:

    command = "" if no_seed else f" dbt seed --full-refresh && "

    if macros:
        command += f"dbt run-operation {macros}"
    else:
        command += f"dbt {cmd}"
        if models:
            command += " --models " + " ".join(models)
        if full_refresh:
            command += " --full-refresh"
        if not no_fail_fast:
            command += " --fail-fast"

    if not persist:
        command += " && dbt run-operation drop_branch_schemas"

    return command


def _long_cycle(
    cmd: str,
    persist: Optional[bool] = False,
    no_fail_fast: Optional[bool] = False,
    full_refresh: Optional[bool] = False,
    no_seed: Optional[bool] = False,
    select: Optional[Tuple] = (),
    models: Optional[Tuple] = (),
    macros: Optional[Tuple] = (),
) -> str:
    seed_cmd = "" if no_seed else "&& dbt seed --full-refresh"
    command = f"dbt clean && dbt deps {seed_cmd}"

    if macros:
        run_operation = " && dbt run-operation "
        command += run_operation + run_operation.join(macros)
    else:
        command += f" && dbt {cmd}"
        if select and not no_seed:
            command += " --select " + " ".join(select)
        if models:
            command += " --models " + " ".join(models)
        if full_refresh:
            command += " --full-refresh"
        if not no_fail_fast:
            command += " --fail-fast"

    if not persist:
        command += " && dbt run-operation drop_branch_schemas"

    return command


def dbt_env_vars(branch: str) -> Dict:
    return {
        'PDP_DEV_SCHEMA': _generate_schema_from_branch(branch),
        'PDP_ENV': 'DEVELOPMENT',  # Deprecated - this will be removed!
        'PALM_DBT_ENV': 'DEVELOPMENT',
    }


def _generate_schema_from_branch(branch: str) -> str:
    """Formats the branch name as a schema."""
    user = local_user_lookup()
    return re.sub(r"[^0-9a-zA-Z]+", "_", f"{user}_{branch}").strip("_").lower()



import click
from typing import Optional, Tuple
from palm.plugins.dbt.dbt_palm_utils import dbt_env_vars


@click.command("snapshot")
@click.option(
    "--persist", is_flag=True, help="will not drop the test schema at the end"
)
@click.option("--select", multiple=True)
@click.option("--fast", is_flag=True, help="will skip clean/deps/seed")
@click.pass_context
def cli(ctx, persist: bool, fast: bool, select: Optional[Tuple] = tuple()):
    """Executes the DBT snapshots."""

    if fast:
        cmd = "dbt snapshot"
    else:
        cmd = "dbt clean && dbt deps && dbt snapshot"
    if select:
        cmd += f" --select " + " ".join(select)
    if not persist:
        cmd += " && dbt run-operation drop_branch_schemas"

    env_vars = dbt_env_vars(ctx.obj.palm.branch)
    success, msg = ctx.obj.run_in_docker(cmd, env_vars)
    click.secho(msg, fg="green" if success else "red")



import click
from typing import Optional, Tuple
from palm.plugins.dbt.dbt_palm_utils import dbt_env_vars


@click.command("compile")
@click.option("--models", multiple=True, help="see dbt docs on models flag")
@click.option("--fast", is_flag=True, help="will skip clean/deps/seed")
@click.pass_context
def cli(ctx, fast: bool, models: Optional[Tuple] = tuple()):
    """Cleans up target directory and dependencies, then compiles dbt"""

    if fast:
        cmd = "dbt compile"
    else:
        cmd = "dbt clean && dbt deps && dbt compile"
    if models:
        cmd += f" --models " + " ".join(models)

    env_vars = dbt_env_vars(ctx.obj.palm.branch)
    success, msg = ctx.obj.run_in_docker(cmd, env_vars)
    click.secho(msg, fg="green" if success else "red")



import click
from typing import Optional, Tuple
from palm.plugins.dbt.dbt_palm_utils import shell_options, dbt_env_vars


@click.command("run")
@click.option("--fast", is_flag=True, help="will skip clean/deps/seed")
@click.option(
    "--no-fail-fast",
    is_flag=True,
    help="Disables the setting which exits immediately if a single model fails to build",
)
@click.option(
    "--persist", is_flag=True, help="will not drop the test schema at the end"
)
@click.option("--models", multiple=True, help="see dbt docs on models flag")
@click.option("--select", multiple=True, help="see dbt docs on select flag")
@click.option(
    "--macros", multiple=True, help="see dbt docs on run operations and macros"
)
@click.option(
    "--full-refresh",
    is_flag=True,
    help="will perform a full refresh on incremental models",
)
@click.option("--no-seed", is_flag=True, help="will skip seed full refresh")
@click.pass_context
def cli(
    ctx,
    fast: bool,
    no_fail_fast: bool,
    persist: bool,
    full_refresh: bool,
    no_seed: bool,
    models: Optional[Tuple] = tuple(),
    select: Optional[Tuple] = tuple(),
    macros: Optional[Tuple] = tuple(),
):
    """Runs the DBT repo."""

    cmd = shell_options("run", **locals())
    env_vars = dbt_env_vars(ctx.obj.palm.branch)
    success, msg = ctx.obj.run_in_docker(cmd, env_vars)
    click.secho(msg, fg="green" if success else "red")
